@@ -30,8 +30,8 @@ data ApiGatewayRequest body = ApiGatewayRequest
   { apiGatewayRequestResource              :: !Text
   , apiGatewayRequestPath                  :: !Text
   , apiGatewayRequestHttpMethod            :: !Text
-  , apiGatewayRequestHeaders               :: !(HashMap Text Text)
-  , apiGatewayRequestQueryStringParameters :: !(Maybe [(Text, Maybe Text)])
+  , apiGatewayRequestHeaders               :: !(Maybe (HashMap Text Text))
+  , apiGatewayRequestQueryStringParameters :: !(Maybe (HashMap Text Text))
   , apiGatewayRequestPathParameters        :: !(Maybe (HashMap Text Text))
   , apiGatewayRequestStageVariables        :: !(Maybe (HashMap Text Text))
   , apiGatewayRequestIsBase64Encoded       :: !Bool
