@@ -3,7 +3,7 @@ module Aws.Lambda.Utilities (toJSONText) where
 import Data.Aeson
 import qualified Data.ByteString.Lazy.Char8 as LazyByteString
 import Data.Text
-import qualified Data.Text as T
+import qualified Data.Text.Encoding as T
 
 toJSONText :: ToJSON a => a -> Text
-toJSONText = T.pack . LazyByteString.unpack . encode
\ No newline at end of file
+toJSONText = T.decodeUtf8 . LazyByteString.toStrict . encode
